#include <windows.h>
#include <stdio.h>
#include <memory.h>
#include <malloc.h>

#include "gpulocal.h"

#ifndef DIB_RGB_COLORS
#define DIB_RGB_COLORS  0
#endif

typedef struct {
    BITMAPINFO bi16;
    BITMAPINFO bi24;
    LPSTR lpBits;
    HBITMAP hbm;
} DIB;

static DIB dib;

#if FRAMESKIP
static int framecount=0;
static int frameskip=3;
#endif

void win_resize(int w,int h)
{
        RECT window,client;
        int ew,eh;

        GetWindowRect(hwnd,&window);
        GetClientRect(hwnd,&client);
        ew = (window.right-window.left)-(client.right-client.left),
        eh = (window.bottom-window.top)-(client.bottom-client.top),

        MoveWindow(hwnd,window.left,window.top,w+ew,h+eh,TRUE);
}

void SetDIBHeader(BITMAPINFO *lpbi,int width,int height,int bits)
{
	BITMAPINFOHEADER *lpbih = &lpbi->bmiHeader;
	lpbih->biSize   = sizeof(BITMAPINFOHEADER);
	lpbih->biWidth  = width;
	lpbih->biHeight = -height;
	lpbih->biPlanes = 1;
	lpbih->biBitCount = bits;
	lpbih->biCompression = BI_RGB;
	lpbih->biSizeImage = 0; // (((((width*bits)+7)/8)+3)&~3)*height;
	lpbih->biXPelsPerMeter = 0;
	lpbih->biYPelsPerMeter = 0;
	lpbih->biClrUsed = 0;
	lpbih->biClrImportant = 0;
}


void StretchDIB(HDC hdc,DIB *dib,int x,int y,int w,int h)
{
    HDC     memDC;
    HBITMAP hbmOld;

    if (gpu.status & 0x00200000)
    {
        x = x*2/3;

        SetDIBitsToDevice(hdc,0,0,w,h,x,y,y,y+h,
            dib->lpBits,&dib->bi24,DIB_RGB_COLORS);

        return;
    }

    memDC = CreateCompatibleDC(hdc);
    hbmOld = SelectObject(memDC,dib->hbm);

    BitBlt(hdc,0,0,w,h,memDC,x,y,SRCCOPY);

    SelectObject(memDC,hbmOld);
    DeleteDC(memDC);
}


#if FRAMESKIP==2
extern void (**primfunc);
extern void (*primfuncYES[256])();
extern void (*primfuncNULL[256])();
#endif

void screen_update()
{
    HDC         hdc;

    if (vram) {
        hdc = GetDC(hwnd);
#if FRAMESKIP==2
        if (++framecount == frameskip)
            primfunc=primfuncYES;
        if (framecount > frameskip)
#endif

#if FRAMESKIP==1
        if (++framecount > frameskip)
#endif
            StretchDIB(hdc,&dib,
                       gpu.disp.x, gpu.disp.y,
                       gpu.disp.w, gpu.disp.h);

#if FRAMESKIP
        if (framecount > frameskip)
            {
                framecount = 0;
#if FRAMESKIP==2
                primfunc=primfuncNULL;
#endif
            }
#endif
        ReleaseDC(hwnd,hdc);
    }
}

void *win_createbitmap(int w,int h,int bits)
{
    HDC hdc;

    SetDIBHeader(&dib.bi16,w,h,16);
    SetDIBHeader(&dib.bi24,w*2/3,h,24);

    hdc = GetDC(hwnd);
    dib.hbm = CreateDIBSection(hdc,(BITMAPINFO*)&dib.bi16,DIB_RGB_COLORS,(void**)&dib.lpBits,NULL,0);
    if (dib.hbm==NULL) MessageBox(hwnd,"createdib false",NULL,MB_OK);
    ReleaseDC(hwnd,hdc);

    memset(dib.lpBits,0,dib.bi16.bmiHeader.biSizeImage);

    return dib.lpBits;
}

void win_destroy(void)
{
    DeleteObject(dib.hbm);
}
